@@ -13,7 +13,7 @@ from meutils.pipe import *
 from sklearn.metrics import *
 
 from aizoo.tuner.base import optuna, Tuner
-from aizoo.tab.models import LGBMClassifier
+from aizoo.tab.models import LGBMOOF
 
 
 class F1Optimizer(Tuner):
@@ -38,9 +38,9 @@ class LGBOptimizer(Tuner):
         self.y = y
         self.feval = feval
 
-    def objective(self, trial: optuna.trial.Trial):
+    def objective(self, trial: optuna.trial.Trial, task='Classifier'):
         params = self.trial_choice(trial)
-        _ = LGBMClassifier(params).run(self.X, self.y, feval=self.feval)
+        _ = LGBMOOF(params, task=task).fit(self.X, self.y, feval=self.feval)
         return _
 
 
